@@ -27,7 +27,7 @@ use Carp;
 
 # NOTE: due to CPAN version checks this cannot currently be changed to a
 # standard version string, i.e. '0.21'
-our $VERSION = '35';
+our $VERSION = '36';
 our $fast_mode = undef;
 our $strict_mode = undef;
 
@@ -180,8 +180,31 @@ sub parse {
 	        $append = undef;
 	    }
 	
+            if ( $line =~ /include_file\s*=\s*([\w\-\/\\\:\.]+)/ )
+	    {	my $incfile = $1;
+		$self->parse($incfile);
+		next;
+	    }
+            if ( $line =~ /include_dir\s*=\s*([\w\-\/\\\:\.]+)/ )
+	    {	my $incdir = $1;
+		opendir(INCDIR, $incdir) or next; #Just ignore if can't open directory?
+		my @files = readdir(INCDIR);
+		closedir(INCDIR);
+		@files = grep /\.cfg$/, @files; #Only want *.cfg files
+		@files = grep { -f "$incdir/$_" } @files; #Only want files
+		foreach my $file (@files)
+		{	$self->parse($file);
+		}
+		next;
+	    }
+
 	    # end of object definition
-	    if ( $line =~ /}(.*)$/ ) {
+	    # Some object attributes are strings, which can contain a right-curly bracket and confuse this parser:
+	    #  - The proper fix would be to make the parser sensitive to arbitrary string attributes, but I will just
+	    #    do it the easy way for now and assume there is no more text on the same line after the right-curly
+	    #    bracket that closes the object definition.
+	    #if ( $line =~ /}(.*)$/ ) {
+	    if ( $line =~ /}(\s*)$/ ) {
 	        $in_definition = undef;
             # continue parsing after closing object with text following the '}'
             $append = $1;
@@ -531,7 +554,15 @@ sub register_object_list {
     my( $self, $object, $attribute, $attr_type ) = @_;
 
     # split on comma surrounded by whitespace or by just whitespace
-    my @to_find = split /\s*,\s*|\s+/, $object->$attribute();
+    #  - don't try splitting it if it has already been split by the Nagios::Object::_set function!
+    #  - same bug reported in CPAN's RT:  http://rt.cpan.org/Public/Bug/Display.html?id=31291
+    my @to_find;
+    my $value = $object->$attribute();
+    if (ref $value eq 'ARRAY') {
+	@to_find = @{ $value };
+    } else {
+	@to_find = split /\s*,\s*|\s+/, $value;
+    }
     my @found = ();
 
     # handle splat '*' matching of all objects of a type (optimization)
